@@ -13,8 +13,8 @@ sub _croak {
 
 use XSLoader;
 BEGIN {
-    our $VERSION = '2.000001-TRIAL';
-    $VERSION =~ s/-TRIAL[0-9]*\z//s;
+    our $VERSION = '2.000004-TRIAL';
+    $VERSION =~ s/-TRIAL[0-9]*\z//;
     XSLoader::load;
 }
 
@@ -169,14 +169,18 @@ my %type_map = (
 
     around             => {
         defaults    => 'method',
+        name        => 'required',
         install_sub => 'around',
         shift       => ['$orig', '$self'],
+        runtime     => 1,
     },
     (
         map +(
             $_ => {
                 defaults    => 'method',
+                name        => 'required',
                 install_sub => $_,
+                runtime     => 1,
             }
         ), qw(
             before after augment override
@@ -684,8 +688,18 @@ value, write C<$param = undef>. There is a special shortcut syntax for
 this case: C<$param = undef> can also be written C<$param => (with no following
 expression).
 
+ fun foo($x = undef, $y = undef, $z = undef) {
+     # three arguments, all optional
+     ...
+ }
+
+ fun foo($x=, $y=, $z=) {
+     # shorter syntax, same meaning
+     ...
+ }
+
 Optional parameters must come at the end. It is not possible to have a required
-parameter following an optional one.
+parameter after an optional one.
 
 =head3 Slurpy/rest parameters
 
@@ -926,11 +940,9 @@ C<before>, C<after>, C<around>, C<augment>, or C<override> function
 
 works like
 
- BEGIN {
-     &before('foo', method ($x, $y, $z) {
-         ...
-     });
- }
+ &before('foo', method ($x, $y, $z) {
+     ...
+ });
 
 C<after>, C<augment>, and C<override> work the same way.
 
@@ -943,18 +955,36 @@ C<@_> into C<$self> (as C<method> does), it shifts off I<two> values:
 
 works like
 
- BEGIN {
-     &around('foo', sub :method {
-         my $orig = shift;
-         my $self = shift;
-         my ($x, $y, $z) = @_;
-         ...
-     });
- }
+ &around('foo', sub :method {
+     my $orig = shift;
+     my $self = shift;
+     my ($x, $y, $z) = @_;
+     ...
+ });
 
 (except you also get the usual C<Function::Parameters> features such as
 checking the number of arguments, etc).
 
+C<$orig> and C<$self> both count as invocants and you can override their names
+like this:
+
+ around foo($original, $object: $x, $y, $z) {
+     # $original is a reference to the wrapped method;
+     # $object is the object we're being called on
+     ...
+ }
+
+If you use C<:> to pick your own invocant names in the parameter list of
+C<around>, you must specify exactly two variables.
+
+These modifiers also differ from C<fun> and C<method> (and C<sub>) in that they
+require a function name (there are no anonymous method modifiers) and they
+take effect at runtime, not compile time. When you say C<fun foo() {}>, the
+C<foo> function is defined right after the closing C<}> of the function body is
+parsed. But with e.g. C<before foo() {}>, the declaration becomes a normal
+function call (to the C<before> function in the current package), which is
+performed at runtime.
+
 =head3 Prototypes and attributes
 
 You can specify attributes (see L<perlsub/Subroutine Attributes>) for your
@@ -994,7 +1024,7 @@ a wrapper module that enables C<Function::Parameters> automatically, just call
 C<< Function::Parameters->import >> from your own C<import> method (and
 C<< Function::Parameters->unimport >> from your C<unimport>, as required).
 
-=head3 Gory details about importing
+=head3 Gory details of importing
 
 At the lowest layer C<use Function::Parameters ...> takes a list of one or more
 hash references. Each key is a keyword to be defined as specified by the
@@ -1277,10 +1307,13 @@ Equivalent to:
      defaults    => 'method',
      install_sub => 'around',
      shift       => ['$orig', '$self'],
+     runtime     => 1,
+     name        => 'required',
  }
 
 i.e. just like L<C<method>|/C<method>> but with a custom installer
-(C<'around'>) and two implicit first parameters.
+(C<'around'>), two implicit first parameters, only taking effect at
+runtime, and a method name is required.
 
 =item C<before>
 
@@ -1289,10 +1322,12 @@ Equivalent to:
  {
      defaults    => 'method',
      install_sub => 'before',
+     runtime     => 1,
+     name        => 'required',
  }
 
 i.e. just like L<C<method>|/C<method>> but with a custom installer
-(C<'before'>).
+(C<'before'>), only taking effect at runtime, and a method name is required.
 
 =item C<after>
 
@@ -1301,10 +1336,12 @@ Equivalent to:
  {
      defaults    => 'method',
      install_sub => 'after',
+     runtime     => 1,
+     name        => 'required',
  }
 
 i.e. just like L<C<method>|/C<method>> but with a custom installer
-(C<'after'>).
+(C<'after'>), only taking effect at runtime, and a method name is required.
 
 =item C<augment>
 
@@ -1313,10 +1350,12 @@ Equivalent to:
  {
      defaults    => 'method',
      install_sub => 'augment',
+     runtime     => 1,
+     name        => 'required',
  }
 
 i.e. just like L<C<method>|/C<method>> but with a custom installer
-(C<'augment'>).
+(C<'augment'>), only taking effect at runtime, and a method name is required.
 
 =item C<override>
 
@@ -1325,10 +1364,12 @@ Equivalent to:
  {
      defaults    => 'method',
      install_sub => 'override',
+     runtime     => 1,
+     name        => 'required',
  }
 
 i.e. just like L<C<method>|/C<method>> but with a custom installer
-(C<'override'>).
+(C<'override'>), only taking effect at runtime, and a method name is required.
 
 =back
 
@@ -1412,6 +1453,85 @@ Equivalent to C<< 'before', 'after', 'around', 'augment', 'override' >>.
 
 =back
 
+For example, when you say
+
+ use Function::Parameters qw(:modifiers);
+
+C<:modifiers> is an import tag that L<expands to|/C<':modifiers'>>
+
+ use Function::Parameters qw(before after around augment override);
+
+Each of those is another import tag. Stepping through the first one:
+
+ use Function::Parameters qw(before);
+
+is L<equivalent to|/C<'before'>>:
+
+ use Function::Parameters { before => 'before' };
+
+This says to define the keyword C<before> according to the
+L<configuration bundle C<before>|/C<before>>:
+
+ use Function::Parameters {
+     before => {
+         defaults    => 'method',
+         install_sub => 'before',
+         runtime     => 1,
+         name        => 'required',
+     },
+ };
+
+The C<< defaults => 'method' >> part L<pulls in|/Configuration bundles> the
+contents of the L<C<'method'> configuration bundle|/C<method>> (which is the
+same as L<C<'method_strict'>|/C<method_strict>>):
+
+ use Function::Parameters {
+     before => {
+         defaults    => 'function_strict',
+         attributes  => ':method',
+         shift       => '$self',
+         invocant    => 1,
+         install_sub => 'before',
+         runtime     => 1,
+         name        => 'required',
+     },
+ };
+
+This in turn uses the
+L<C<'function_strict'> configuration bundle|/C<function_strict>> (which is
+empty because it consists of default values only):
+
+ use Function::Parameters {
+     before => {
+         attributes  => ':method',
+         shift       => '$self',
+         invocant    => 1,
+         install_sub => 'before',
+         runtime     => 1,
+         name        => 'required',
+     },
+ };
+
+But if we wanted to be completely explicit, we could write this as:
+
+ use Function::Parameters {
+     before => {
+         check_argument_count => 1,
+         check_argument_types => 1,
+         default_arguments    => 1,
+         named_parameters     => 1,
+         reify_type           => 'auto',
+         types                => 1,
+
+         attributes  => ':method',
+         shift       => '$self',
+         invocant    => 1,
+         install_sub => 'before',
+         runtime     => 1,
+         name        => 'required',
+     },
+ };
+
 =head2 Incompatibilites with version 1 of C<Function::Parameters>
 
 =over
@@ -1429,7 +1549,8 @@ C<Function::Parameters> is loaded.)
 
 Parameter lists used to be optional. The syntax C<fun foo { ... }> would accept
 any number of arguments. This syntax has been removed; you now have to write
-C<fun foo(@) { ... }> to accept (and ignore) all arguments.
+C<fun foo(@) { ... }> to accept (and ignore) all arguments. On the other hand,
+if you meant for the function to take no arguments, write C<fun foo() { ... }>.
 
 =item *
 
@@ -1452,6 +1573,21 @@ second argument has been removed. (Technically this is a core perl bug
 (L<RT #129239|https://rt.perl.org/Public/Bug/Display.html?id=129239>) that
 wasn't so much fixed as worked around in C<Function::Parameters>.)
 
+If you want your type reifier to be compatible with both versions, you can do
+this:
+
+ sub my_reifier {
+     my ($type, $package) = @_;
+     $package //= caller;
+     ...
+ }
+
+Or using C<Function::Parameters> itself:
+
+ fun my_reifier($type, $package = caller) {
+     ...
+ }
+
 =back
 
 =head1 SUPPORT AND DOCUMENTATION
